************dfadjustSE*********************
*Gregory A. Bruich, Ph.D. and Rene Livas
*Department of Economics, Harvard University
*Send corrections and suggestions to gbruich@fas.haravard.edu and rlivas@g.harvard.edu

*Program to calculate a bias-adjusted robust variance estimator with degrees of freedom correction for linear regression with independent observations
*For details, see Imbens and Kolesár (2016), Bell and McCaffrey (2002), Sattherthwaite (1946)

capture program drop dfadjustSE
program define dfadjustSE, rclass
	syntax varlist(min=2 fv) [if] [, coef(integer 1)] [REPORTREG]
	cap postclose dfadjustSE
	
	preserve
	
	*Separate dependent from regressors
	local depvar: word 1 of `varlist'
	local indepvars: list varlist -depvar
	
	*Define for which variable to report calculations
	if `coef' != 0{
		local kvar: word `coef' of `indepvars'
		}
	else {
		local kvar = "_cons"
	}
	
	*Store the diagonal elements of the projection matrix
	tempvar _pii _pii_sqrt
	quietly reg `varlist' `if'
	quietly keep if e(sample)
	quietly predict `_pii' , hat
	quietly gen `_pii_sqrt' = 1/(sqrt(1-`_pii'))
	mata: pii_sqrt = st_data(.,"`_pii_sqrt'")

	*Get the HC2 standard errors
	if !missing("`reportreg'") {
		reg `varlist' `if', hc2
	}
	else{
		quietly reg `varlist' `if', hc2
	}
	
	*Store data set size and number of covariates
	local K = colsof(e(b))
	local N = e(N)
	local se_hc2 = _se[`kvar']
	
	*Store model matrix
	tempvar const 
	cap gen `const' = 1
	mata: Var = st_data(., "`const' `indepvars'") 

	*Define vector that pulls out coefficient we want
	mata: ek=J(`K',1,0) 
	mata: ek[`coef'+1,1]=1

	*QR decomp
	mata: H = tau = R1 = .
	mata: hqrd(Var, H, tau, R1)
	mata: Q1 = hqrdq1(H, tau)

	*AQ
	mata: AQ = pii_sqrt:*Q1
	mata: a = AQ * qrsolve(R1', ek)
	mata: B = a:*Q1

	*Calculate Degrees of Freedom
	mata: k_bm = (sum(a:^2) - sum(B:^2)):^2 / (sum(a:^4)-2*sum((a:*B):^2)+sum(cross(B,B):^2))

	*Save results
	mata: st_numscalar("r(k_bm)", k_bm)
	local k_bm = r(k_bm)
	
	*Store results so that they can be accessed using r()
	return scalar k_bm = `k_bm'
	return scalar t_k_bm = invt(`k_bm',0.975)
	return scalar adjse = `se_hc2'*invt(`k_bm',0.975)/1.96
	return scalar hc2_se = `se_hc2'
	
	display "---------------------------------------------------"
	display "Calculations for estimated coefficient on `kvar'"
	display "Bell and McCaffrey (2002) degrees of freedom K:"  
	display %12.3f `k_bm'
	display "0.975 quantile for t(dof = K) distribution:" 
	display %12.3f  invt(`k_bm',0.975)
	display "Imbens and Kolesár (2016) adjusted HC2 standard error:" 
	display%12.4f `se_hc2'*invt(`k_bm',0.975)/1.96
	display "---------------------------------------------------"
	
	restore
	
end
*End program

